@@ -1,11 +1,24 @@
-var npa = require('../npa.js')
-var path = require('path')
-var os = require('os')
+const path = require('path').posix
+const os = require('os')
 
-require('tap').test('basic', function (t) {
+const normalizePath = p => p && p.replace(/^[a-zA-Z]:/, '').replace(/\\/g, '/')
+
+const cwd = normalizePath(process.cwd())
+process.cwd = () => cwd
+const normalizePaths = spec => {
+  spec.saveSpec = normalizePath(spec.saveSpec)
+  spec.fetchSpec = normalizePath(spec.fetchSpec)
+  return spec
+}
+
+const t = require('tap')
+const npa = t.mock('../npa.js', { path })
+t.on('bailout', () => process.exit(1))
+
+t.test('basic', function (t) {
   t.setMaxListeners(999)
 
-  var tests = {
+  const tests = {
     'foo@1.2': {
       name: 'foo',
       escapedName: 'foo',
@@ -13,7 +26,7 @@ require('tap').test('basic', function (t) {
       saveSpec: null,
       fetchSpec: '1.2',
       raw: 'foo@1.2',
-      rawSpec: '1.2'
+      rawSpec: '1.2',
     },
 
     'foo@~1.2': {
@@ -23,7 +36,7 @@ require('tap').test('basic', function (t) {
       saveSpec: null,
       fetchSpec: '~1.2',
       raw: 'foo@~1.2',
-      rawSpec: '~1.2'
+      rawSpec: '~1.2',
     },
 
     '@foo/bar': {
@@ -34,7 +47,7 @@ require('tap').test('basic', function (t) {
       rawSpec: '',
       saveSpec: null,
       fetchSpec: 'latest',
-      type: 'tag'
+      type: 'tag',
     },
 
     '@foo/bar@': {
@@ -45,7 +58,7 @@ require('tap').test('basic', function (t) {
       rawSpec: '',
       saveSpec: null,
       fetchSpec: 'latest',
-      type: 'tag'
+      type: 'tag',
     },
 
     '@foo/bar@baz': {
@@ -56,7 +69,7 @@ require('tap').test('basic', function (t) {
       rawSpec: 'baz',
       saveSpec: null,
       fetchSpec: 'baz',
-      type: 'tag'
+      type: 'tag',
     },
 
     '@f fo o al/ a d s ;f': {
@@ -66,7 +79,7 @@ require('tap').test('basic', function (t) {
       rawSpec: '@f fo o al/ a d s ;f',
       saveSpec: 'file:@f fo o al/ a d s ;f',
       fetchSpec: '/test/a/b/@f fo o al/ a d s ;f',
-      type: 'directory'
+      type: 'directory',
     },
 
     'foo@1.2.3': {
@@ -75,7 +88,7 @@ require('tap').test('basic', function (t) {
       type: 'version',
       saveSpec: null,
       fetchSpec: '1.2.3',
-      raw: 'foo@1.2.3'
+      raw: 'foo@1.2.3',
     },
 
     'foo@=v1.2.3': {
@@ -85,7 +98,7 @@ require('tap').test('basic', function (t) {
       saveSpec: null,
       fetchSpec: '=v1.2.3',
       raw: 'foo@=v1.2.3',
-      rawSpec: '=v1.2.3'
+      rawSpec: '=v1.2.3',
     },
 
     'foo@npm:bar': {
@@ -104,8 +117,8 @@ require('tap').test('basic', function (t) {
         raw: 'bar',
         rawSpec: '',
         saveSpec: null,
-        fetchSpec: 'latest'
-      }
+        fetchSpec: 'latest',
+      },
     },
 
     'git+ssh://git@notgithub.com/user/foo#1.2.3': {
@@ -115,7 +128,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://git@notgithub.com/user/foo#1.2.3',
       fetchSpec: 'ssh://git@notgithub.com/user/foo',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://git@notgithub.com/user/foo#1.2.3'
+      raw: 'git+ssh://git@notgithub.com/user/foo#1.2.3',
     },
 
     'git+ssh://git@notgithub.com/user/foo': {
@@ -125,7 +138,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://git@notgithub.com/user/foo',
       fetchSpec: 'ssh://git@notgithub.com/user/foo',
       gitCommittish: null,
-      raw: 'git+ssh://git@notgithub.com/user/foo'
+      raw: 'git+ssh://git@notgithub.com/user/foo',
     },
 
     'git+ssh://git@notgithub.com:user/foo': {
@@ -135,7 +148,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://git@notgithub.com:user/foo',
       fetchSpec: 'git@notgithub.com:user/foo',
       gitCommittish: null,
-      raw: 'git+ssh://git@notgithub.com:user/foo'
+      raw: 'git+ssh://git@notgithub.com:user/foo',
     },
 
     'git+ssh://mydomain.com:foo': {
@@ -145,7 +158,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:foo',
       fetchSpec: 'mydomain.com:foo',
       gitCommittish: null,
-      raw: 'git+ssh://mydomain.com:foo'
+      raw: 'git+ssh://mydomain.com:foo',
     },
 
     'git+ssh://git@notgithub.com:user/foo#1.2.3': {
@@ -155,7 +168,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://git@notgithub.com:user/foo#1.2.3',
       fetchSpec: 'git@notgithub.com:user/foo',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://git@notgithub.com:user/foo#1.2.3'
+      raw: 'git+ssh://git@notgithub.com:user/foo#1.2.3',
     },
 
     'git+ssh://mydomain.com:foo#1.2.3': {
@@ -165,7 +178,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:foo#1.2.3',
       fetchSpec: 'mydomain.com:foo',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://mydomain.com:foo#1.2.3'
+      raw: 'git+ssh://mydomain.com:foo#1.2.3',
     },
 
     'git+ssh://mydomain.com:foo/bar#1.2.3': {
@@ -175,7 +188,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:foo/bar#1.2.3',
       fetchSpec: 'mydomain.com:foo/bar',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://mydomain.com:foo/bar#1.2.3'
+      raw: 'git+ssh://mydomain.com:foo/bar#1.2.3',
     },
 
     'git+ssh://mydomain.com:1234#1.2.3': {
@@ -185,7 +198,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:1234#1.2.3',
       fetchSpec: 'ssh://mydomain.com:1234',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://mydomain.com:1234#1.2.3'
+      raw: 'git+ssh://mydomain.com:1234#1.2.3',
     },
 
     'git+ssh://mydomain.com:1234/hey#1.2.3': {
@@ -195,7 +208,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:1234/hey#1.2.3',
       fetchSpec: 'ssh://mydomain.com:1234/hey',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://mydomain.com:1234/hey#1.2.3'
+      raw: 'git+ssh://mydomain.com:1234/hey#1.2.3',
     },
 
     'git+ssh://mydomain.com:1234/hey': {
@@ -205,7 +218,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://mydomain.com:1234/hey',
       fetchSpec: 'ssh://mydomain.com:1234/hey',
       gitCommittish: null,
-      raw: 'git+ssh://mydomain.com:1234/hey'
+      raw: 'git+ssh://mydomain.com:1234/hey',
     },
 
     'git+ssh://username:password@mydomain.com:1234/hey#1.2.3': {
@@ -215,7 +228,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://username:password@mydomain.com:1234/hey#1.2.3',
       fetchSpec: 'ssh://username:password@mydomain.com:1234/hey',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://username:password@mydomain.com:1234/hey#1.2.3'
+      raw: 'git+ssh://username:password@mydomain.com:1234/hey#1.2.3',
     },
 
     'git+ssh://git@github.com/user/foo#1.2.3': {
@@ -225,7 +238,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://git@github.com/user/foo.git#1.2.3',
       fetchSpec: 'ssh://git@github.com/user/foo.git',
       gitCommittish: '1.2.3',
-      raw: 'git+ssh://git@github.com/user/foo#1.2.3'
+      raw: 'git+ssh://git@github.com/user/foo#1.2.3',
     },
 
     'git+ssh://git@notgithub.com/user/foo#semver:^1.2.3': {
@@ -237,7 +250,7 @@ require('tap').test('basic', function (t) {
       fetchSpec: 'ssh://git@notgithub.com/user/foo',
       gitCommittish: null,
       gitRange: '^1.2.3',
-      raw: 'git+ssh://git@notgithub.com/user/foo#semver:^1.2.3'
+      raw: 'git+ssh://git@notgithub.com/user/foo#semver:^1.2.3',
     },
 
     'git+ssh://git@notgithub.com:user/foo#semver:^1.2.3': {
@@ -249,7 +262,7 @@ require('tap').test('basic', function (t) {
       fetchSpec: 'git@notgithub.com:user/foo',
       gitCommittish: null,
       gitRange: '^1.2.3',
-      raw: 'git+ssh://git@notgithub.com:user/foo#semver:^1.2.3'
+      raw: 'git+ssh://git@notgithub.com:user/foo#semver:^1.2.3',
     },
 
     'git+ssh://git@github.com/user/foo#semver:^1.2.3': {
@@ -260,7 +273,7 @@ require('tap').test('basic', function (t) {
       fetchSpec: 'ssh://git@github.com/user/foo.git',
       gitCommittish: null,
       gitRange: '^1.2.3',
-      raw: 'git+ssh://git@github.com/user/foo#semver:^1.2.3'
+      raw: 'git+ssh://git@github.com/user/foo#semver:^1.2.3',
     },
 
     'git+ssh://git@github.com:user/foo#semver:^1.2.3': {
@@ -271,7 +284,7 @@ require('tap').test('basic', function (t) {
       fetchSpec: 'ssh://git@github.com/user/foo.git',
       gitCommittish: null,
       gitRange: '^1.2.3',
-      raw: 'git+ssh://git@github.com:user/foo#semver:^1.2.3'
+      raw: 'git+ssh://git@github.com:user/foo#semver:^1.2.3',
     },
 
     'user/foo#semver:^1.2.3': {
@@ -282,7 +295,7 @@ require('tap').test('basic', function (t) {
       fetchSpec: null,
       gitCommittish: null,
       gitRange: '^1.2.3',
-      raw: 'user/foo#semver:^1.2.3'
+      raw: 'user/foo#semver:^1.2.3',
     },
 
     'git+file://path/to/repo#1.2.3': {
@@ -292,7 +305,7 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+file://path/to/repo#1.2.3',
       fetchSpec: 'file://path/to/repo',
       gitCommittish: '1.2.3',
-      raw: 'git+file://path/to/repo#1.2.3'
+      raw: 'git+file://path/to/repo#1.2.3',
     },
 
     'git://notgithub.com/user/foo': {
@@ -301,7 +314,7 @@ require('tap').test('basic', function (t) {
       type: 'git',
       saveSpec: 'git://notgithub.com/user/foo',
       fetchSpec: 'git://notgithub.com/user/foo',
-      raw: 'git://notgithub.com/user/foo'
+      raw: 'git://notgithub.com/user/foo',
     },
 
     '@foo/bar@git+ssh://notgithub.com/user/foo': {
@@ -311,7 +324,27 @@ require('tap').test('basic', function (t) {
       saveSpec: 'git+ssh://notgithub.com/user/foo',
       fetchSpec: 'ssh://notgithub.com/user/foo',
       rawSpec: 'git+ssh://notgithub.com/user/foo',
-      raw: '@foo/bar@git+ssh://notgithub.com/user/foo'
+      raw: '@foo/bar@git+ssh://notgithub.com/user/foo',
+    },
+
+    'git@npm:not-git': {
+      name: 'git',
+      type: 'alias',
+      subSpec: {
+        type: 'tag',
+        registry: true,
+        name: 'not-git',
+        fetchSpec: 'latest',
+      },
+      raw: 'git@npm:not-git',
+    },
+
+    'not-git@hostname.com:some/repo': {
+      name: null,
+      type: 'git',
+      saveSpec: 'git+ssh://not-git@hostname.com:some/repo',
+      fetchSpec: 'not-git@hostname.com:some/repo',
+      raw: 'not-git@hostname.com:some/repo',
     },
 
     '/path/to/foo': {
@@ -319,8 +352,8 @@ require('tap').test('basic', function (t) {
       escapedName: null,
       type: 'directory',
       saveSpec: 'file:/path/to/foo',
-      fetchSpec: path.resolve(__dirname, '/path/to/foo'),
-      raw: '/path/to/foo'
+      fetchSpec: '/path/to/foo',
+      raw: '/path/to/foo',
     },
 
     '/path/to/foo.tar': {
@@ -328,8 +361,8 @@ require('tap').test('basic', function (t) {
       escapedName: null,
       type: 'file',
       saveSpec: 'file:/path/to/foo.tar',
-      fetchSpec: path.resolve(__dirname, '/path/to/foo.tar'),
-      raw: '/path/to/foo.tar'
+      fetchSpec: '/path/to/foo.tar',
+      raw: '/path/to/foo.tar',
     },
 
     '/path/to/foo.tgz': {
@@ -337,8 +370,8 @@ require('tap').test('basic', function (t) {
       escapedName: null,
       type: 'file',
       saveSpec: 'file:/path/to/foo.tgz',
-      fetchSpec: path.resolve(__dirname, '/path/to/foo.tgz'),
-      raw: '/path/to/foo.tgz'
+      fetchSpec: '/path/to/foo.tgz',
+      raw: '/path/to/foo.tgz',
     },
     'file:path/to/foo': {
       name: null,
@@ -346,7 +379,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:path/to/foo',
       fetchSpec: '/test/a/b/path/to/foo',
-      raw: 'file:path/to/foo'
+      raw: 'file:path/to/foo',
     },
     'file:path/to/foo.tar.gz': {
       name: null,
@@ -354,7 +387,7 @@ require('tap').test('basic', function (t) {
       type: 'file',
       saveSpec: 'file:path/to/foo',
       fetchSpec: '/test/a/b/path/to/foo.tar.gz',
-      raw: 'file:path/to/foo.tar.gz'
+      raw: 'file:path/to/foo.tar.gz',
     },
 
     'file:~/path/to/foo': {
@@ -362,8 +395,8 @@ require('tap').test('basic', function (t) {
       escapedName: null,
       type: 'directory',
       saveSpec: 'file:~/path/to/foo',
-      fetchSpec: os.homedir().replace(/\\/g, '/') + '/path/to/foo',
-      raw: 'file:~/path/to/foo'
+      fetchSpec: normalizePath(path.join(os.homedir(), '/path/to/foo')),
+      raw: 'file:~/path/to/foo',
     },
 
     'file:/~/path/to/foo': {
@@ -371,8 +404,26 @@ require('tap').test('basic', function (t) {
       escapedName: null,
       type: 'directory',
       saveSpec: 'file:~/path/to/foo',
-      fetchSpec: os.homedir().replace(/\\/g, '/') + '/path/to/foo',
-      raw: 'file:/~/path/to/foo'
+      fetchSpec: normalizePath(path.join(os.homedir(), '/path/to/foo')),
+      raw: 'file:/~/path/to/foo',
+    },
+
+    'file:/~path/to/foo': {
+      name: null,
+      escapedName: null,
+      type: 'directory',
+      saveSpec: 'file:/~path/to/foo',
+      fetchSpec: '/~path/to/foo',
+      raw: 'file:/~path/to/foo',
+    },
+
+    'file:/.path/to/foo': {
+      name: null,
+      escapedName: null,
+      type: 'directory',
+      saveSpec: 'file:/.path/to/foo',
+      fetchSpec: '/.path/to/foo',
+      raw: 'file:/.path/to/foo',
     },
 
     'file:../path/to/foo': {
@@ -381,7 +432,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:../path/to/foo',
       fetchSpec: '/test/a/path/to/foo',
-      raw: 'file:../path/to/foo'
+      raw: 'file:../path/to/foo',
     },
 
     'file:/../path/to/foo': {
@@ -390,7 +441,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:../path/to/foo',
       fetchSpec: '/test/a/path/to/foo',
-      raw: 'file:/../path/to/foo'
+      raw: 'file:/../path/to/foo',
     },
 
     'file:///path/to/foo': {
@@ -399,7 +450,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:/path/to/foo',
       fetchSpec: '/path/to/foo',
-      raw: 'file:///path/to/foo'
+      raw: 'file:///path/to/foo',
     },
     'file:/path/to/foo': {
       name: null,
@@ -407,7 +458,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:/path/to/foo',
       fetchSpec: '/path/to/foo',
-      raw: 'file:/path/to/foo'
+      raw: 'file:/path/to/foo',
     },
     'file://path/to/foo': {
       name: null,
@@ -415,7 +466,7 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:/path/to/foo',
       fetchSpec: '/path/to/foo',
-      raw: 'file://path/to/foo'
+      raw: 'file://path/to/foo',
     },
     'file:////path/to/foo': {
       name: null,
@@ -423,7 +474,15 @@ require('tap').test('basic', function (t) {
       type: 'directory',
       saveSpec: 'file:/path/to/foo',
       fetchSpec: '/path/to/foo',
-      raw: 'file:////path/to/foo'
+      raw: 'file:////path/to/foo',
+    },
+
+    'file://.': {
+      name: null,
+      escapedName: null,
+      type: 'directory',
+      saveSpec: 'file:',
+      raw: 'file://.',
     },
 
     'http://insecure.com/foo.tgz': {
@@ -432,7 +491,7 @@ require('tap').test('basic', function (t) {
       type: 'remote',
       saveSpec: 'http://insecure.com/foo.tgz',
       fetchSpec: 'http://insecure.com/foo.tgz',
-      raw: 'http://insecure.com/foo.tgz'
+      raw: 'http://insecure.com/foo.tgz',
     },
 
     'https://server.com/foo.tgz': {
@@ -441,7 +500,7 @@ require('tap').test('basic', function (t) {
       type: 'remote',
       saveSpec: 'https://server.com/foo.tgz',
       fetchSpec: 'https://server.com/foo.tgz',
-      raw: 'https://server.com/foo.tgz'
+      raw: 'https://server.com/foo.tgz',
     },
 
     'foo@latest': {
@@ -450,7 +509,7 @@ require('tap').test('basic', function (t) {
       type: 'tag',
       saveSpec: null,
       fetchSpec: 'latest',
-      raw: 'foo@latest'
+      raw: 'foo@latest',
     },
 
     foo: {
@@ -459,20 +518,59 @@ require('tap').test('basic', function (t) {
       type: 'tag',
       saveSpec: null,
       fetchSpec: 'latest',
-      raw: 'foo'
-    }
+      raw: 'foo',
+    },
+
+    'foo@ 1.2 ': {
+      name: 'foo',
+      escapedName: 'foo',
+      type: 'range',
+      saveSpec: null,
+      fetchSpec: '1.2',
+      raw: 'foo@ 1.2 ',
+      rawSpec: ' 1.2 ',
+    },
+
+    'foo@ 1.2.3 ': {
+      name: 'foo',
+      escapedName: 'foo',
+      type: 'version',
+      saveSpec: null,
+      fetchSpec: '1.2.3',
+      raw: 'foo@ 1.2.3 ',
+      rawSpec: ' 1.2.3 ',
+    },
+
+    'foo@1.2.3 ': {
+      name: 'foo',
+      escapedName: 'foo',
+      type: 'version',
+      saveSpec: null,
+      fetchSpec: '1.2.3',
+      raw: 'foo@1.2.3 ',
+      rawSpec: '1.2.3 ',
+    },
+
+    'foo@ 1.2.3': {
+      name: 'foo',
+      escapedName: 'foo',
+      type: 'version',
+      saveSpec: null,
+      fetchSpec: '1.2.3',
+      raw: 'foo@ 1.2.3',
+      rawSpec: ' 1.2.3',
+    },
   }
 
   Object.keys(tests).forEach(function (arg) {
-    var res = npa(arg, '/test/a/b')
+    const res = normalizePaths(npa(arg, '/test/a/b'))
     t.ok(res instanceof npa.Result, arg + ' is a result')
     Object.keys(tests[arg]).forEach(function (key) {
-      t.has(res[key], tests[arg][key], arg + ' [' + key + ']')
+      t.match(res[key], tests[arg][key], arg + ' [' + key + ']')
     })
-    //    t.has(res, tests[arg], arg + ' matches expectations')
   })
 
-  var objSpec = { name: 'foo', rawSpec: '1.2.3' }
+  let objSpec = { name: 'foo', rawSpec: '1.2.3' }
   t.equal(npa(objSpec, '/whatnot').toString(), 'foo@1.2.3', 'parsed object')
 
   objSpec.where = '/whatnot'
@@ -482,25 +580,29 @@ require('tap').test('basic', function (t) {
     'git+http://foo.com/bar', 'parsed git toString')
 
   objSpec = { raw: './foo/bar', where: '/here' }
-  t.equal(npa(objSpec).fetchSpec, '/here/foo/bar', '`where` is reused')
+  t.equal(normalizePath(npa(objSpec).fetchSpec), '/here/foo/bar', '`where` is reused')
 
-  var res = new npa.Result({ name: 'bar', rawSpec: './foo/bar' })
+  let res = new npa.Result({ name: 'bar', rawSpec: './foo/bar' })
   t.equal(res.toString(), 'bar@./foo/bar', 'toString with only rawSpec')
   res = new npa.Result({ rawSpec: './x/y' })
-  t.equal(res.toString(), './x/y', 'toString with only rawSpec, no name')
+  t.equal(normalizePath(res.toString()), './x/y', 'toString with only rawSpec, no name')
   res = new npa.Result({ rawSpec: '' })
   t.equal(res.toString(), '', 'toString with nothing')
 
   objSpec = { raw: './foo/bar', where: '/here' }
-  t.equal(npa(objSpec, '/whatnot').fetchSpec, '/whatnot/foo/bar', '`where` arg overrides the one in the spec object')
+  t.equal(
+    normalizePath(npa(objSpec, '/whatnot').fetchSpec),
+    '/whatnot/foo/bar',
+    '`where` arg overrides the one in the spec object'
+  )
 
   t.equal(npa(npa('foo@1.2.3')).toString(), 'foo@1.2.3', 'spec is passthrough')
 
-  var parsedSpec = npa('./foo', './here')
+  const parsedSpec = npa('./foo', './here')
   t.equal(npa(parsedSpec), parsedSpec, 'reused if no where')
   t.equal(npa(parsedSpec, './here'), parsedSpec, 'reused if where matches')
-  t.notEqual(npa(parsedSpec, './there'), parsedSpec, 'new instance if where does not match')
-  t.notEqual(npa(parsedSpec, './there').fetchSpec, '/there/foo', 'new instance has new where')
+  t.not(npa(parsedSpec, './there'), parsedSpec, 'new instance if where does not match')
+  t.not(npa(parsedSpec, './there').fetchSpec, '/there/foo', 'new instance has new where')
   // Completely unreasonable invalid garbage throws an error
   t.throws(function () {
     t.comment(npa('this is not a \0 valid package name or url'))
@@ -522,14 +624,72 @@ require('tap').test('basic', function (t) {
     npa('foo@npm:foo/bar')
   }, 'aliases only work for registry deps')
 
-  t.has(npa.resolve('foo', '^1.2.3', '/test/a/b'), { type: 'range' }, 'npa.resolve')
-  t.has(npa.resolve('foo', 'file:foo', '/test/a/b'), { type: 'directory', fetchSpec: '/test/a/b/foo' }, 'npa.resolve file:')
-  t.has(npa.resolve('foo', '../foo/bar', '/test/a/b'), { type: 'directory' }, 'npa.resolve no protocol')
-  t.has(npa.resolve('foo', 'file:../foo/bar', '/test/a/b'), { type: 'directory' }, 'npa.resolve file protocol')
-  t.has(npa.resolve('foo', 'file:../foo/bar.tgz', '/test/a/b'), { type: 'file' }, 'npa.resolve file protocol w/ tgz')
-  t.has(npa.resolve(null, '4.0.0', '/test/a/b'), { type: 'version', name: null }, 'npa.resolve with no name')
-  t.has(npa.resolve('foo', 'file:abc'), { type: 'directory', raw: 'foo@file:abc' }, 'npa.resolve sets raw right')
-  t.has(npa('./path/to/thing/package@1.2.3/'), { name: null, type: 'directory' }, 'npa with path in @ in it')
-  t.has(npa('path/to/thing/package@1.2.3'), { name: null, type: 'directory' }, 'npa w/o leading or trailing slash')
+  t.has(npa.resolve('foo', '^1.2.3', '/test/a/b'), {
+    type: 'range',
+  }, 'npa.resolve')
+  t.has(normalizePaths(npa.resolve('foo', 'file:foo', '/test/a/b')), {
+    type: 'directory',
+    fetchSpec: '/test/a/b/foo',
+  }, 'npa.resolve file:')
+  t.has(npa.resolve('foo', '../foo/bar', '/test/a/b'), {
+    type: 'directory',
+  }, 'npa.resolve no protocol')
+  t.has(npa.resolve('foo', 'file:../foo/bar', '/test/a/b'), {
+    type: 'directory',
+  }, 'npa.resolve file protocol')
+  t.has(npa.resolve('foo', 'file:../foo/bar.tgz', '/test/a/b'), {
+    type: 'file',
+  }, 'npa.resolve file protocol w/ tgz')
+  t.has(npa.resolve(null, '4.0.0', '/test/a/b'), {
+    type: 'version',
+    name: null,
+  }, 'npa.resolve with no name')
+  t.has(npa.resolve('foo', 'file:abc'), {
+    type: 'directory',
+    raw: 'foo@file:abc',
+  }, 'npa.resolve sets raw right')
+  t.has(npa('./path/to/thing/package@1.2.3/'), {
+    name: null,
+    type: 'directory',
+  }, 'npa with path in @ in it')
+  t.has(npa('path/to/thing/package@1.2.3'), {
+    name: null,
+    type: 'directory',
+  }, 'npa w/o leading or trailing slash')
+  t.end()
+})
+
+t.test('strict 8909 compliance mode', t => {
+  t.teardown(() => process.env.NPM_PACKAGE_ARG_8909_STRICT = '0')
+  process.env.NPM_PACKAGE_ARG_8909_STRICT = '1'
+
+  t.throws(() => npa('file://.'), {
+    message: 'Invalid file: URL, must be absolute if // present',
+    raw: 'file://.',
+  })
+
+  t.throws(() => npa('file://some/relative/path'), {
+    message: 'Invalid file: URL, must be absolute if // present',
+    raw: 'file://some/relative/path',
+  })
+
+  // I cannot for the life of me figure out how to make new URL('file:...')
+  // actually fail to parse.  it seems like it accepts any garbage you can
+  // throw at it.  However, because it theoretically CAN throw, here's a test.
+  t.throws(() => {
+    const npa = t.mock('../npa.js', {
+      url: {
+        URL: class {
+          constructor () {
+            throw new Error('thansk i haet it')
+          }
+        },
+      },
+    })
+    npa('file:thansk i haet it')
+  }, {
+    message: 'Invalid file: URL, must comply with RFC 8909',
+  })
+
   t.end()
 })
